/**

        QT4D Write -- A plugin that creates a 4d QuickTime movie
	    from a 4d set of TIFFs.
        
        Some of the basic code was adapted from the MovieWriter plugin written by Wayne Rasband, NIH
        
        The CompressionSettings class was modified from code by Charles Thomas, LOCI project,
        Univ of Wisconsin-Madison.
        
        Progress bar code was adapted from an online course hosted by Dave Marshall at
        http://www.cs.cf.ac.uk/Dave/HCI/HCI_Handout_CALLER/node125.html
        
        The basics of the image compression was adapted from Brad Lowe brad@nuspectra.com
        based on various QT sample code

	Kludgy workaround for bugs in the ImageCompressionDialog.requestSequenceSettings() call in 
	QTJ 6.1/Java 1.4.1 added
	
	v. 0.2 - Sept. 28 2006
 */


import ij.plugin.frame.*;
import ij.*;
import ij.process.*;
import ij.io.*;
import ij.gui.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.Vector;
import java.lang.Number;

import javax.swing.*;
import javax.swing.event.*;

import quicktime.qd.*;
import quicktime.*;
import quicktime.std.*;
import quicktime.io.*;
//import quicktime.sound.*;
import quicktime.std.image.*;
import quicktime.std.movies.*;
import quicktime.std.movies.media.*;
import quicktime.util.*;
//import quicktime.app.display.*;
//import quicktime.app.image.*;
import quicktime.app.view.*;
import quicktime.app.QTFactory;
import quicktime.std.qtcomponents.*;
 
public class QT_4D_Writer extends PlugInFrame implements StdQTConstants, Errors {

	//private QTCanvas canv;
	//private QTImageDrawer qid;
	//private ImagePainter np;
	private int numFrames;
	private int kWidth;
	private int kHeight;
	private File soundFile;
	private String title;
	private ImageStack stack;
        public int myIndex;
	public boolean cancelOperation = false;
        public boolean workaround = false;

	//***** Compressor settings
	static int comp_type = kVideoCodecType;
	static int spatial_quality = codecNormalQuality;
	static float frame_rate = 8;
	static int key_frame_rate = 10;
        public int depth;
        
        //*** Crunch stuff
        QTFile movieDest;	// movie
        int codecQuality = codecNormalQuality;
        CodecName codecName = null;	
        int maxFrames = -1;
        int curFrame = 0;
        QTFile images[];
        boolean flatten = false;
        static String defaultCodecName = "Animation";

        int kVidTimeScale = 600;	// 1 second = 600
        int defaultFrameDuration = 60;	// frames / 600 = 1/10 sec
        static boolean qtOpen = false;
        static String shortVersion = "ImageListToMovie dev 0.1 by brad@nuspectra.com";
        // List of image types to allow for importing.
        // more can be added.. keep lowercase.
        final static String okTypes[] = {".jpg", ".jpeg", ".gif", ".tif", ".tiff", ".bmp", ".png", ".pct"};
        
        int numPlanes = -1;
        double timeInterval = -1;
        double spaceInterval = -1;
        String movieInfo = null;	

						
public QT_4D_Writer () {
		super("Temp");
	}		

	public void run(String arg) {
		//IJ.write("v14");
                /**
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp!=null)
			saveAsMovie(imp);
		else
			IJ.noImage();
		IJ.register( QT_Movie_Writer.class); 
                */
        String mySystemString = System.getProperty("java.version");
	double myQTJVersion = (double) QTBuild.getVersion() +  (double) QTBuild.getSubVersion()/10;
 	//IJ.showMessage("Java VM version: " + mySystemString + "  QT Java version: " + QTBuild.getVersion() + "." + QTBuild.getSubVersion());
	//IJ.showMessage("Offset to version number: " + mySystemString.indexOf("1.4");
	//following updated to account for JVM 1.5
				 	    if (((mySystemString.indexOf("1.4") >= 0) || (mySystemString.indexOf("1.5") >= 0) || (mySystemString.indexOf("1.6") >= 0) || (mySystemString.indexOf("1.7") >= 0)) && (myQTJVersion >= 6.1))
				 	    {
				       		// Systems with QTJ 6.1 & Java VM 1.4 or higher require a workaround for now, so set boolean to true.
				        	workaround = true;
				        }

                saveAsMovie();
	 }

	public void saveAsMovie() {
        //open folder with the images
                String dir = null;
                String foundName = null;
                String dstFile = "out.mov";
                String codec = defaultCodecName;
                int frameDuration = 0;
                Vector myList = new Vector();	// a list of image files to convert
                int quality = -1;
                boolean all = false;
		String infoFileRootName = null;

        	OpenDialog od = new OpenDialog("Select a file in source folder...", "");
		if (od.getFileName()==null) return;
		String dir1 = od.getDirectory();
                //IJ.showStatus("Directory: " + dir1);
                String[] list = new File(dir1).list();
                String[] list3 = new String[list.length];
		if (list==null) return;
                int theCounter = 0;
                
               //IJ.showStatus("Parsing file data in the directory...");
		for (int i=0; i<list.length; i++) {
			//IJ.log((i+1)+": "+list[i]);
			//IJ.showStatus((i+1) +"/"+list.length);
                        //IJ.showProgress((i+1)/list.length);
			File f = new File(dir1+list[i]);
			if (!f.isDirectory()) {
				ImagePlus img = new Opener().openImage(dir1, list[i]);
				if (img!=null) {
                                        foundName = list[i];
                                        //IJ.showMessage(foundName + " is not null!");
                                        break;//stop when we find the first actual image file
				}//img not null
			}//not a directory
		}//for loop
                
                //now we have the name of an image file, which we can use to parse the other files in this dir
             IJ.showStatus("Parsing file data...");
             int myIndex = foundName.lastIndexOf(".");
             //IJ.showMessage("Index of last period: "+myIndex);
             if (myIndex == -1) {
               // IJ.beep();
                IJ.showMessage("Files do not seem to have the correct name structure. Sorry!");
                IJ.showStatus("");
                return;
                }
             String rootName = foundName.substring(0,myIndex);
             String suffixName = foundName.substring(myIndex+1,foundName.length());
            // IJ.log("Root name: " + rootName);
            // IJ.log("Suffix: " + suffixName);
             //test to see of the suffix is an integer. If so, them we have a folder of images with a root name, a period, and sequential numbers
             boolean useNumbersAsSuffix = true;
             try{
                Integer myInt = new Integer(suffixName);
                //IJ.log("The suffix is a number! It is " + myInt.intValue());
             } catch (NumberFormatException e) {
			//IJ.log("The suffix is not a number! It is '"+suffixName + "'");
                        useNumbersAsSuffix = false;
                        }
             //double fileStructure = IJ.getNumber("Enter a number. 1 = .0001; 2 = .jpg: ",1);
             if (!useNumbersAsSuffix) {
                boolean suffixInList = false;
                for (int x=0;x<okTypes.length;x++) {
                    //IJ.showMessage("okType = " + okTypes[x] + "; suffiName = " + suffixName);
                    if (("." + suffixName).equalsIgnoreCase(okTypes[x])) {suffixInList = true;}
                }//for x
                if (!suffixInList) {IJ.showMessage("The file extension " + suffixName + " is not supported. Sorry!"); return;}
                //it was in the list!
                for (int i=0; i<list.length; i++) {
			//IJ.write((i+1)+": "+list[i]);
			//IJ.showStatus((i+1) +"/"+list.length);
                        //IJ.showProgress((i+1)/list.length);
			File f = new File(dir1+list[i]);
			if (!f.isDirectory()) {
				//ImagePlus img = new Opener().openImage(dir1, list[i]);
                                if (list[i].indexOf(suffixName) > -1){
				//if (img!=null) {
					//IJ.write(list[i] + " is not null!");
                                        list3[theCounter] = list[i];
                                        theCounter = theCounter + 1;
                                        IJ.showStatus("Parsing file data...Files found:" + theCounter);
                                     myList.addElement(new QTFile(new File(dir1, list[i])));
                                        
				}//img not null
			}//not a directory
		}//for loop

             }
             
            if (useNumbersAsSuffix){                          
            for (int i=0; i<list.length; i++) {
			//IJ.write((i+1)+": "+list[i]);
			//IJ.showStatus((i+1) +"/"+list.length);
                        //IJ.showProgress((i+1)/list.length);
			File f = new File(dir1+list[i]);
			if (!f.isDirectory()) {
				//ImagePlus img = new Opener().openImage(dir1, list[i]);
                                if ((list[i].indexOf(rootName) > -1) & (list[i].indexOf(".txt") == -1) & (list[i].indexOf(".info") == -1)){
				//if (img!=null) {
					//IJ.write(list[i] + " is not null!");
                                        list3[theCounter] = list[i];
                                        theCounter = theCounter + 1;
                                        IJ.showStatus("Parsing file data...Files found:" + theCounter);
                                     myList.addElement(new QTFile(new File(dir1, list[i])));
                                        
				}//img not null
			}//not a directory
		}//for loop
                
             }//use numbered suffix
                
                IJ.showStatus("Total image files: " + theCounter);
                
                if (useNumbersAsSuffix){
                    //Let's try to find an info file. If there is a file with .txt or .info and
                    //the same root name under the numbered suffix scheme, use it to construct info
                    numPlanes = -1;timeInterval = -1.0; spaceInterval = -1.0;
                    File myPath = new File(dir1);
                    getInfoFile(myPath, rootName);
                }
			else
			{
                    //Let's try to find an info file. If there is a file with .txt or .info, use it to construct info
              // IJ.log("Parsing file data in the directory...");
		  for (int i=0; i<list.length; i++) {
			//IJ.log((i+1)+": "+list[i]);
			if ((list[i].indexOf(".txt") > -1)){
			//IJ.log("Found info file candidate! Name: " + list[i]);
			infoFileRootName = list[i].substring(0,list[i].indexOf(".txt"));
			break;//stop when we find the first candidate info file
			}
		  }//for loop
			//IJ.log("Root name for info file: " + infoFileRootName);
			//IJ.log("Directory: " + dir1);
                    File myPath = new File(dir1);
                    getInfoFile(myPath, infoFileRootName);
		}
               // if (numPlanes > 0) {IJ.showMessage("Number of focal planes based on file info: " + (int)numPlanes);}
                if (numPlanes < 1) {numPlanes = (int) IJ.getNumber("Couldn't get info from file. Enter the number of focal planes: ",1);}
                if (numPlanes < 1) { numPlanes = 1;}
                if (timeInterval < 0) { timeInterval = IJ.getNumber("Enter the number of seconds between time points: ",60);}
                if (spaceInterval < 0) { spaceInterval = IJ.getNumber("Enter the slice spacing in microns between focal planes: ", 1);}
                if (movieInfo == null) { movieInfo = IJ.getString("Enter descriptive information about the movie: ","");}
                
                /*
		   IJ.log("Movie information");
                IJ.log("-------------------");
                IJ.log("Number of focal planes: " + numPlanes);
                IJ.log("Seconds between time points: " + timeInterval);
                IJ.log("Spacing between focal planes in microns: " + spaceInterval);
                IJ.log("Other info: " + movieInfo);
                **/
                
                Vector list2 = new Vector();
                double totalTimePoints = theCounter/numPlanes;
                int k = -1;
                myIndex = 0;
		for (int i=0; i<numPlanes; i++) {
                //IJ.write("Focal plane " + (i+1) + ":");
                for (int j=0; j<totalTimePoints; j++){
                    k = k+1;
                    myIndex = j * numPlanes + i;
                    list2.addElement(new QTFile(new File(dir1, list3[myIndex])));
                    // IJ.showStatus("Adding file " + list3[myIndex]);
                    //list2.add(myList[myIndex]);
                    //IJ.write("list2["+k+"]=" + list2[k]);
                    }
		}//for loop

                
		//stack = imp.getStack();
		//title = img.getTitle();
		try {
			QTSession.open();
			//initialize(myImg);
			//makeMovie();
			//canv.removeClient();
                        // we have a list of image files to convert into a movie
                    if (list2.size() > 0) {
                   // IJ.showMessage("list2 has a non-zero size of " + list2.size());
                        QTFile imageFiles[] = new QTFile[list2.size()];
                        list2.copyInto(imageFiles);
                         //IJ.showMessage("imageFiles has a size of " + imageFiles.length);
                                                Crunch converter = new Crunch(imageFiles);
                        
                        getCodecSettings(imageFiles[0]);
			//IJ.showMessage("cancelOperation = " + cancelOperation);
		if (cancelOperation == true) {return;}
                        //IJ.showMessage("Returned from image compression dialog!");
                        //codec = comp_type;
                        quality = spatial_quality;
						/*
                         IJ.log("");
		         IJ.log("Compressor settings");
                         IJ.log("-------------------");
                         IJ.log("Quality: "+quality);
                         IJ.log("Codec type: "+comp_type);
                         IJ.log("Bit depth: " + depth);
                         IJ.log("Frame rate: " + frame_rate);
                         IJ.log("Key frame rate: "+key_frame_rate);
						 **/
                        if (quality>=0) converter.setQuality(quality);
                        //converter.setCodec(codec);
                        converter.setCodec(comp_type);
                        frameDuration = (int) ((1/frame_rate)*600);
                        if (frameDuration > 0) converter.setFrameDuration(frameDuration);
                        title = "untitled";
                        SaveDialog sd = new SaveDialog("Save QuickTime Movie as...", title, ".mov");
			dir = sd.getDirectory();
			String name = sd.getFileName();
			if(name == null)
				throw new QTIOException (userCanceledErr, "");
			dstFile= dir+name;
			/*
                         IJ.log("");
                         IJ.log("Compressing image sequence...(this may take a while!)");
                         IJ.log("");
			**/
                        if (!all) converter.convert(new QTFile(dstFile));
                        
                        writeInfoFile(dstFile+".txt");
                    }

			QTSession.close();
			dispose();
                        
		} catch (NoClassDefFoundError e) {
			IJ.error("QuickTime for Java required");
		} catch (Exception e) {
			QTSession.close();
		}
	
        }//saveAsMovie
        
        void writeInfoFile(String myFile){
        
                //write the data to an info file - Line 1 = general info;
                //Line 2 = focal planes; Line 3 = space between focal planes
                //Line 4 = time interval
                
                FileOutputStream out; // declare a file output object
                PrintStream p; // declare a print stream object

                try
                {
                        // Create a new file output stream
                        // connected to "myfile.txt"
                        out = new FileOutputStream(myFile);

                        // Connect print stream to the output stream
                        p = new PrintStream( out );
		
                        p.println (movieInfo); p.println(numPlanes);
                        p.println(spaceInterval); p.println(timeInterval);

                        p.close();
                }
                catch (Exception e)
                {
                        System.err.println ("Error writing to file");
                }
                
            }

        
        
        
        void printStackTrace(Exception e) {
		CharArrayWriter caw = new CharArrayWriter();
		PrintWriter pw = new PrintWriter(caw);
		e.printStackTrace(pw);
		IJ.write(caw.toString());
	}

        
     public void getInfoFile(File thePath, String theName){
     //test for existence of file first!!
     boolean infoFileExists = false;
     boolean txtFileExists = false;
     boolean infotxtFileExists = false;
	String str = "";
        String focalPlaneString = "";
        String timeIntervalString = "";
        String spaceIntervalString = "";
	String myName = "";
     myName = theName + ".info";
     File myFile = new File(thePath,myName);
     File myFile1 = myFile;
    //IJ.write("Abs path:"+myFile.getAbsolutePath());
    if (myFile1.exists()) {infoFileExists = true;}
    myName = theName + ".txt";
     File myFile2 = new File(thePath,myName);
    if (myFile2.exists()) {txtFileExists = true;}
     myName = theName + ".info.txt";
     File myFile3 = new File(thePath,myName);
    if (myFile3.exists()) {infotxtFileExists = true;}
    if (infotxtFileExists) myFile = myFile3;
    if (txtFileExists) myFile = myFile2;    
    if (infoFileExists) myFile = myFile1;
    // IJ.log("Final file name: " + myFile);   
     if (myFile.exists()) {
          try {
    // IJ.log("Text file name (made it to try): " + myFile);
	FileReader myFileReader = new FileReader(myFile);
        BufferedReader myReader = new BufferedReader(myFileReader);
        //String str = "";
        //String focalPlaneString = "";
		
		/*
        IJ.log("Information for data set: ");
	IJ.log ("-------------------------");
	*/
        int lineCount = 0;
        while ((str = myReader.readLine()) != null) {
        lineCount = lineCount + 1;
            //IJ.write("Line " + lineCount + ": " + str);
            if (lineCount == 1) {/**IJ.write("Information: " + str);*/movieInfo = str.trim();}
            if (lineCount == 2) {/**IJ.write("Focal planes: " + str);*/focalPlaneString = str;} 
            if (lineCount == 3) {/**IJ.write("Spacing (microns): " + str);*/spaceIntervalString = str;}
            if (lineCount == 4) {/**IJ.write("Time interval (sec): " + str);*/timeIntervalString = str;}
        }
        myFileReader.close();

        int tempPlanes = Integer.parseInt(focalPlaneString.trim());
        Double tempSpace = new Double(spaceIntervalString.trim());
        Double tempTime = new Double(timeIntervalString.trim());
	
        //IJ.write("Focal planes: " + tempPlanes);
        numPlanes = tempPlanes;
        spaceInterval = tempSpace.doubleValue();
        timeInterval = tempTime.doubleValue();


//IJ.write("Info: " + movieInfo);
/**movieInfo = "Here's some sample info!";
numPlanes = 1;
spaceInterval = 1.0;
timeInterval = 45.0;
*/
        } catch (IOException e) {
        printStackTrace(e);
        //IJ.showMessage("Focal plane string: " + focalPlaneString);
       }     

     
     } else {
     numPlanes = -1;
     //IJ.beep();
     //IJ.write("There is no associated text file!");
     //IJ.write("File I was looking for:");
    // IJ.write(myFile.getAbsolutePath());
     
     }
    
    }

        
        void getCodecSettings(QTFile test_file) throws Exception {
		ImageCompressionDialog comp_dlog = null;
		SpatialSettings spatial_settings = null;
		TemporalSettings temporal_settings = null;
		DataRateSettings data_rate_settings = null;
		QDRect test_rect = null;
		int color_depth = 0; // let ICM choose depth
		CodecComponent comp = CodecComponent.anyCodec;
		int temporal_quality = codecNormalQuality;
		int data_rate = 1000;
		int frame_duration = 8;
		boolean cancelled = false;
		int test_flags = 0;
		int flags = 0;
                PixMap test_pix_map = null;
                
		
		//***** Get the standard QT compression dialog
		comp_dlog = new ImageCompressionDialog();
		//***** Set the default image
                    
                //***** Set the default image
                //comp_dlog.setTestImagePixMap(test_pix_map, test_rect, test_flags);
                 //IJ.showMessage("File: "+ test_file.getName());
                 
                 GraphicsImporter myImporter = new GraphicsImporter(test_file);
                 comp_dlog.setTestImagePict(myImporter.getAsPicture(), test_rect, test_flags);
                 
                                                       		   
		//***** Set up the default settings
		spatial_settings = new SpatialSettings(comp_type, comp, color_depth, spatial_quality);
		comp_dlog.setInfoSpatialSettings(spatial_settings);
		temporal_settings = new TemporalSettings(temporal_quality, frame_rate, key_frame_rate);
		comp_dlog.setInfoTemporalSettings(temporal_settings);
		data_rate_settings = new DataRateSettings(data_rate, frame_duration, codecMinQuality, codecMinQuality);
		comp_dlog.setInfoDataRateSettings(data_rate_settings);
		
		//***** Set the default such that BEST DEPTH is not an option
		flags &= ~scShowBestDepth;
		comp_dlog.setInfoPreferences(flags);

	cancelOperation = true;

		//***** Display the dialog for the user
		try {
			//requestSequenceSettings is broken with QTJ 6.1 & Java VM 1.4.1 -- needs a silly workaround
			//comp_dlog.requestSequenceSettings();
                      if (workaround) {comp_dlog.requestImageSettings();} else {comp_dlog.requestSequenceSettings();}
		} catch (Error er) {
			throw new Error("User cancelled");
		}
			
		if (!cancelled) {
			//***** Get the user settings
			spatial_settings = comp_dlog.getInfoSpatialSettings();
                        depth = spatial_settings.getDepth();
			comp_type = spatial_settings.getCodecType();
			spatial_quality = spatial_settings.getSpatialQuality();	
                        //following is broken!
                        if (!workaround){	
			temporal_settings = comp_dlog.getInfoTemporalSettings();
			frame_rate = temporal_settings.getFrameRate();
                       key_frame_rate = temporal_settings.getKeyFrameRate();
                       cancelOperation = false;
                        }
                        //use dialog to circumvent
                        //IJ.showMessage("Cancelled: " + cancelled + " workaround: " + workaround);
		if ((!cancelled) && (workaround)) {
                GenericDialog gd = new GenericDialog("Frame Rate Options", IJ.getInstance());
                double xinitial = 8.0;
                double yinitial = 0.0;
                gd.addNumericField("Frame rate (fps):",xinitial, 1);
                gd.addNumericField("Key frames:", yinitial,0);
		gd.showDialog();
		if (gd.wasCanceled()) {cancelOperation = true;} else{		
                        frame_rate = (float) gd.getNextNumber();
                        key_frame_rate = (int) gd.getNextNumber();
			cancelOperation = false;
			//IJ.showMessage("Cancel = false");
			}
		}//not cancelled

              }//not cancelled but need workaround
                        
	}

        
public class Crunch{

QTFile images[];
boolean stopProgram;
JFrame myFrame;
JLabel myLabel;
JProgressBar progress;

public Crunch(QTFile images[]) throws QTException
{
init(images);
}

protected void init(QTFile images[]) throws QTException
{
//openQT();
this.images = images;
}

public void setCodec(int myCType) throws Exception
{
CodecNameList list = new CodecNameList(1);
for (int i=0;i<list.getCount();i++)
{
CodecName item = list.getNth(i);
if (myCType == item.getCType())
{
codecName=item;
return;
}
}
}

// spatial and temporal quality. 
public void setQuality(int i)
{
codecQuality=i;
}

/**
public void setCodec(int zIndex) throws Exception
{
CodecNameList list = new CodecNameList(1);
codecName = list.getNth(zIndex);
}
*/
public void setFrameDuration(int i)
{
defaultFrameDuration = i;
}


public int getMovieFrameCount() throws Exception
{
return images.length;
}

protected QTFile getNextImage() throws Exception
{
//IJ.showStatus("Compressing frame " + (curFrame+1) + " of " + images.length);
//IJ.showProgress((curFrame+1)/images.length);
return images[curFrame];
}


public void convert(File movieDest) throws Exception
{
convert(new QTFile(movieDest));
}

public void convert(QTFile movieDest) throws Exception        
{
//IJ.showMessage("Codec name: " + codecName.getName());
if (codecName == null) setCodec(kVideoCodecType);
stopProgram = false;

Movie theMovie = Movie.createMovieFile (movieDest,
kMoviePlayer, 
createMovieFileDeleteCurFile | createMovieFileDontCreateResFile);

//make window for display progress and cancel button

        progress = new JProgressBar();
	JButton         button = new JButton();
        //JLabel          myLabel = new JLabel();
        JPanel          topPanel = new JPanel();

               myFrame = new JFrame("Compressing images...");
               //myLabel = new Label("");
      
      
		myFrame.setSize( 310, 130 );
		//setBackground( Color.gray );

		topPanel = new JPanel();
		topPanel.setPreferredSize( new Dimension( 310, 130 ) );
		myFrame.getContentPane().add( topPanel );

		// Create a label and progress bar
		myLabel = new JLabel( "" );
		myLabel.setPreferredSize( new Dimension( 280, 24 ) );
                myLabel.setForeground(Color.black);
		topPanel.add( myLabel );

		progress = new JProgressBar();
		progress.setPreferredSize( new Dimension( 300, 20 ) );
		progress.setMinimum( 0 );
		progress.setMaximum( 90 );
		progress.setValue( 0 );
		progress.setBounds( 20, 35, 260, 20 );
                progress.setForeground(Color.blue);
		topPanel.add( progress );

		button = new JButton( "Stop Compression" );
		topPanel.add( button );
		button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                    stopProgram = true;
                    myFrame.dispose();
                    }
      
      
      
     // myFrame.add(myLabel, BorderLayout.CENTER);
      //Button b1 = new Button("Stop Compression");
     // b1.addActionListener(new ActionListener() {
     //    public void actionPerformed(ActionEvent e) {
     //       stopProgram = true;
      //      myFrame.dispose();
      //   }
      });
     // myFrame.add(b1, BorderLayout.SOUTH);
     // myFrame.setSize(200, 300);
      myFrame.show();

//

try 
{	
addVideoTrack( theMovie );

OpenMovieFile outStream = OpenMovieFile.asWrite (movieDest); 
theMovie.addResource( outStream, movieInDataForkResID, movieDest.getName() );
outStream.close();

if (flatten)
{
// msTimer timer = new msTimer();
theMovie.flatten(0, movieDest, kMoviePlayer, IOConstants.smSystemScript, createMovieFileDeleteCurFile | createMovieFileDontCreateResFile,
movieInDataForkResID, movieDest.getName());
// System.out.println (timer.reportString("flatten movie"));
}
myFrame.dispose();
IJ.showStatus ("Finished movie. Duration:"+theMovie.getDuration() + " Length:"+movieDest.length());

theMovie.disposeQTObject();
theMovie = null;
}
catch (Exception qte) {
qte.printStackTrace(); 

if (movieDest.exists())
movieDest.delete();
throw qte;
}

}

int getFrameDuration()
{
return defaultFrameDuration;	
}

private void addVideoTrack (Movie theMovie) throws QTException, Exception 
{
int numFrames = getMovieFrameCount();
if (maxFrames > 0 && numFrames > maxFrames)	
numFrames=maxFrames;
progress.setMaximum(numFrames);//set the max for the progress bar
QDRect rect=null;
QDGraphics gw=null;
Track vidTrack=null;
VideoMedia vidMedia=null;
QTHandle imageHandle=null;
RawEncodedImage compressedImage=null;
CSequence seq=null;
ImageDescription desc=null;

//IJ.write ("Compressing:" + numFrames + " frames");


for (curFrame = 0; curFrame < numFrames; curFrame++) 
{
//IJ.log("f#:" + curFrame + "; about to get image...");
QTFile imageFile = getNextImage();
//IJ.log("f#:" + curFrame + "; got image...");
// import the image into QuickTime
GraphicsImporter myGraphicsImporter = new GraphicsImporter (imageFile);
GraphicsImporterDrawer drawer = new GraphicsImporterDrawer (myGraphicsImporter);

if (curFrame == 0)
{
/* Do initial setup, now that we can find out the width, height of first image */

rect = myGraphicsImporter.getBoundsRect();
gw = new QDGraphics (rect);
vidTrack = theMovie.addTrack (rect.getWidth(), rect.getHeight(), 0);	// no volume
vidMedia = new VideoMedia (vidTrack, kVidTimeScale); 
vidMedia.beginEdits();

int size = QTImage.getMaxCompressionSize (gw, 
rect, 
/*gw.getPixMap().getPixelSize(),**/depth,
codecQuality, 
codecName.getCType(), 
codecName.getCodecComponent());

//IJ.log("Size: " + size);

imageHandle = new QTHandle (size, true);
imageHandle.lock();
compressedImage = RawEncodedImage.fromQTHandle(imageHandle);
seq = new CSequence (gw,
rect, 
/*gw.getPixMap().getPixelSize(),**/depth,
codecName.getCType(), 
codecName.getCodecComponent(),	
codecQuality, 
codecQuality, 
QT_4D_Writer.key_frame_rate,
//numFrames,	//1 key frame ********NEEDS TO BE CHANGED!!!!!
null, //cTab,
0);

desc = seq.getDescription();
}

myLabel.setText("Compressing frame " +  (curFrame+1) + " of " + numFrames);
progress.setValue( curFrame + 1 );

if (stopProgram) {
break;
} 

drawer.setGWorld (gw);
drawer.redraw (null);

CompressedFrameInfo info = seq.compressFrame (gw, 
rect, 
codecFlagUpdatePrevious, 
compressedImage);

boolean isKeyFrame = info.getSimilarity() == 0;

//if (curFrame % 10 == 0)
//IJ.write("f#:" + curFrame + " kf=" + isKeyFrame + " sim=" + info.getSimilarity());
//IJ.log("f#:" + curFrame + "; about to add sample...");
vidMedia.addSample (imageHandle, 
0, // dataOffset,
info.getDataSize(),
getFrameDuration(), // desired time per frame	
desc,
1, // one sample
(isKeyFrame ? 0 : mediaSampleNotSync)); // no flags
//IJ.log("f#:" + curFrame + "; added sample...");
myGraphicsImporter.disposeQTObject();
//IJ.log("f#:" + curFrame + "; disposed QTObject...");
}

vidMedia.endEdits();

int kTrackStart	= 0;
int kMediaTime = 0;
int kMediaRate	= 1;
vidTrack.insertMedia (kTrackStart, kMediaTime, vidMedia.getDuration(), kMediaRate);
}

}
}
